program myclient1;

{$mode objfpc}

uses
  JS, Classes, SysUtils, Web, DB, JSONDataset, ExtJSDataset, restconnection;


type

 { TRestDataset }

 TRestDataset = Class(TExtJSJSONObjectDataSet)
 private
   FConnection: TRestConnection;
 Protected
   Function DoGetDataProxy: TDataProxy; override;
 Public
   Property Connection: TRestConnection Read FConnection Write FConnection;
 end;

{ TDemoForm }

 TDemoForm=class
 private
  finSupplier  : TJSElement;
  finProduct   : TJSElement;
  finType      : TJSElement;
  finSize      : TJSElement;
  finReference : TJSElement;
  finPackage   : TJSElement;
  fSearchPanel : TJSelement;
  fBodyPanel   : TJSelement;

  fBtnSearch   : TJSelement;
  fBtnClear    : TJSelement;

  fTable       : TJSElement;
  fBody        : TJSElement;

  fDS          : TRestDataset;
  fConn        : TRestConnection;

  procedure AddRecords;
  function CreateRow(aDS: TRestDataset): TJSElement;
  function GetValue(aID: string): string;
public
  constructor create;

  function btnClearClick(aEvent{%H-}: TJSMouseEvent): boolean;
  function btnSearchClick(aEvent{%H-}: TJSMouseEvent): boolean;
  function CreateRow(AName: String): TJSElement;
  function CreateTable: TJSElement;

  procedure DoAfterLoad(DataSet{%H-}: TDataSet);
  procedure DSOpen(DataSet{%H-}: TDataSet);

  procedure DoGetURL(Sender: TComponent; aRequest: TDataRequest; Var aURL: String);
  procedure DoLoadFail(DataSet{%H-}: TDataSet; ID{%H-}: Integer; const ErrorMsg: String);


  function CreateInput(aParent: TJSElement; Atype, AName, ALabel, aID: String): TJSHTMLInputElement;
end;

function TRestDataset.DoGetDataProxy: TDataProxy;
begin
  Result:=Connection.DataProxy;
end;

procedure TDemoForm.DoGetURL(Sender: TComponent; aRequest: TDataRequest; Var aURL: String);
begin
  aURL:='http://localhost:3000/packages';
end;

procedure TDemoForm.DoLoadFail(DataSet: TDataSet; ID: Integer; const ErrorMsg: String);
Var
  N : TJSElement;
begin
  N:=Document.CreateElement('div');
  N.appendChild(Document.createTextNode('Failed to load countries...'+ErrorMsg));
  fBodyPanel.append(N);
end;


function TDemoForm.CreateInput(aParent : TJSElement; Atype,AName,ALabel,aID : String) : TJSHTMLInputElement;
var aDiv,aLabelDiv : TJSElement;
begin
  adiv:=document.createElement('div');
  adiv.className:='form-group';
  aLabelDiv:=document.createElement('label');
  aLabelDiv['for']:=aID;
  aLabelDiv.innerText:=ALabel;
  aDiv.appendChild(aLabelDiv);
  Result:=TJSHTMLInputElement(document.createElement('input'));
  Result.id:=AID;
  Result.name:=AName;
  Result['type']:=AType;
  aDiv.appendChild(Result);
  aParent.appendChild(aDiv);
end;

constructor TDemoForm.create;
var F : TJSelement;
begin
  fSearchPanel  := document.createElement('div');
  fSearchPanel.className := 'searchpanel';
  document.body.appendChild(fSearchPanel);
    F:=document.createElement('form');
  F.ClassName:='form-inline';
  fSearchPanel.appendChild(F);
  finSupplier  := CreateInput(f,'text','ESupplier','Supplier','fSupplier');
  finProduct   := CreateInput(f,'text','EProduct','Product','fProduct');
  finType      := CreateInput(f,'text','EType','Type','fType');
  finSize      := CreateInput(f,'text','ESize','Size','fSize');
  finReference := CreateInput(f,'text','EReference','Reference','fReference');
  finPackage   := CreateInput(f,'text','EPackage','Package','fPackage');
  fBtnSearch   := document.createElement('button');
  fBtnSearch.innerHTML := 'Search';
  fBtnSearch['id'] := 'btnSearch';
  TJSHTMLElement(fBtnSearch).onclick := @btnSearchClick;
  fSearchPanel.appendChild(fBtnSearch);
  fBtnClear   := document.createElement('button');
  fBtnClear.innerHTML := 'Clear';
  fBtnClear['id'] := 'btnClear';
  TJSHTMLElement(fBtnClear).onclick := @btnClearClick;
  fSearchPanel.appendChild(fBtnClear);

  fDS           :=TRestDataset.Create(Nil);
  fConn         :=TRestConnection.Create(nil);
  fConn.BaseURL :='/packages';
  fConn.OnGetURL:=@DoGetURL;
  fDS.Connection:=fConn;
  fDS.OnLoadFail:=@DoLoadFail;
  fDS.AfterLoad :=@DoAfterLoad;
  fDS.AfterOpen :=@DSOpen;
end;

function TDemoForm.CreateTable : TJSElement;
var TH,R,H : TJSElement;

procedure Columnheader(var R,H : TJSElement; const aName : string);
begin
  H := document.createElement('th');
  R.Append(H);
  H.AppendChild(document.createTextNode(aName));
end;

begin
  Result:=document.createElement('table');
//  Result.className:='table table-striped table-bordered table-hover table-condensed';
  TH:=document.createElement('thead');
  Result.Append(TH);
  R:=document.createElement('tr');
  TH.Append(R);
  ColumnHeader(R, H, 'supplier');
  ColumnHeader(R, H, 'product');
  ColumnHeader(R, H, 'type');
  ColumnHeader(R, H, 'size');
  ColumnHeader(R, H, 'quantity');
  ColumnHeader(R, H, 'reference');
  ColumnHeader(R, H, 'package');
  ColumnHeader(R, H, 'pkquantity');
  ColumnHeader(R, H, 'location');
end;

procedure TDemoForm.DoAfterLoad(DataSet: TDataSet);
begin
  if Dataset.Active then
    begin
    Writeln('Loading additional records to table');
    // We're on the last record of the last batch, so move forward 1 record
    Dataset.Next;
    AddRecords;
    end;
end;

procedure TDemoForm.AddRecords;

begin
  While not fDS.EOF do
    begin
    // Writeln(DS.RecNo, ' - ',DS.FieldByName('Name').AsString,'-',DS.FieldByName('Population').AsInteger);
    fBody.Append(CreateRow(fDS));
    fDS.Next;
    end;
end;


function TDemoForm.CreateRow(aDS : TRestDataset) : TJSElement;

  function Columnbody(const aValue : string)  : TJSElement;
  var C : TJSElement;

  begin
    C:=document.createElement('td');
    Result.Append(C);
    C.appendChild(Document.createTextNode(aValue));
  end;

begin
  Result:=document.createElement('tr');

  result.append(Columnbody(aDs.FieldByName('supplier').AsString));
  result.append(Columnbody(aDs.FieldByName('product').AsString));
  result.append(Columnbody(aDs.FieldByName('type').AsString));
  result.append(Columnbody(aDs.FieldByName('size').AsString));
  result.append(Columnbody(aDs.FieldByName('reference').AsString));
  result.append(Columnbody(aDs.FieldByName('package').AsString));
  result.append(Columnbody(aDs.FieldByName('quantity').AsString));
  result.append(Columnbody(aDs.FieldByName('pkquantity').AsString));
  result.append(Columnbody(aDs.FieldByName('location').AsString));
end;

procedure TDemoForm.DSOpen(DataSet: TDataSet);

begin
  console.log('Dataset opened');
  if not Assigned(fTable) then
    begin
    fTable:=CreateTable;
    fBodyPanel.append(fTable);
    fBody:=document.createElement('tbody');
    fTable.Append(fBody);
    end;
  fDS.First;
  AddRecords;
end;


function TDemoForm.btnClearClick(aEvent: TJSMouseEvent): boolean;
var MyLabel : TJSHTMLInputElement;
begin
  MyLabel := TJSHTMLInputElement(document.getElementById('fSupplier'));
  MyLabel.value := 'hallo';
  result := true;
end;

function TDemoForm.btnSearchClick(aEvent: TJSMouseEvent): boolean;
begin
  if Assigned(fBody) then
    fBody.innerHTML:='';
  fDS.Load([],Nil);
  Result:=true;
end;

function TDemoForm.GetValue(aID : string) : string;
var MyLabel : TJSHTMLInputElement;
begin
  MyLabel := TJSHTMLInputElement(document.getElementById(aID));
  result := MyLabel.value;
end;


function TDemoForm.CreateRow(AName : String) : TJSElement;
var C : TJSElement;
begin
  Result:=document.createElement('TR');
  C:=document.createElement('TD');
  Result.Append(C);
  C.appendChild(Document.createTextNode(AName));
end;


begin
  // Your code here
  TDemoForm.create
end.
