unit restdata;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, httpdefs;

Procedure HandleRest(aRequest : TRequest; AResponse : TResponse);



implementation

uses jsonparser,fpjson;

Var
  Packet : TJSONObject;
//  LastID : integer;

Function Data : TJSONArray;

begin
  Result:=Packet.Arrays['Data'];
end;

Function MetaData : TJSONObject;
begin
  Result:=Packet.Objects['metaData'];
end;

Procedure LoadRestData;

Var
  F : TFileStream;
  D : TJSONData;
//  I : Integer;

begin
  F:=TFileStream.Create('fust.json',fmOpenRead or fmShareDenyWrite);
  try
    D:=GetJSON(F);
    if not (D is TJSONObject) then
      D.Free
    else
      Packet:=D as TJSONObject;
    MetaData.Arrays['fields'].Add(TJSONObject.Create(
    ['supplier',
    'product',
    'type',
    'size',
    'reference',
    'package',
    'quantity',
     'pkquantity',
     'location',
     'int']));
  finally
    F.Free;
  end;
end;

Procedure DoneRestData;

begin
  Packet.Free;
end;

Procedure SendRecords(aFirst,aLast : integer; AResponse : TResponse);

Var
  J: TJSONObject;
  A : TJSONArray;
  I : Integer;
begin

  A:=TJSONArray.Create;
  J:=TJSONObject.Create(['metaData',MetaData.Clone,'Data',A]);
  try
    I:=AFirst;
    While (I<=ALast) and (I<Data.Count) do
    begin
      A.Add(Data[i].Clone);
      Inc(I);
    end;
    AResponse.Content:=J.FormatJSON();

    writeln('response json:');
    writeln(AResponse.Content);
    AResponse.ContentLength:=Length(AResponse.Content);
    writeln('length of content : ', AResponse.ContentLength);
    AResponse.ContentType:='application/json';
    AResponse.SendContent;
  finally
    J.Free;
  end;
end;

Function IndexOfID(ID : Integer) : Integer;

begin
  Result:=Data.Count-1;
  While (Result>=0) and (Data.Objects[Result].Integers['id']<>ID) do
   Dec(Result);

end;

Procedure NotFound(AResponse : TResponse);

begin
  AResponse.Code:=404;
  AResponse.CodeText:='Not found';
  AResponse.SendResponse;
end;

Procedure InvalidParam(AResponse : TResponse; AContent : String = '');

begin
  AResponse.Code:=400;
  AResponse.CodeText:='Bad request';
  AResponse.COntent:=AContent;
  AResponse.ContentLength:=Length(AContent);
  AResponse.SendResponse;
end;

Function GetRequestID(aRequest : TRequest) : Integer;

Var
  P : String;
  I : integer;

begin
  P:=ARequest.PathInfo;
  if (P<>'') and (P[1]='/') then
    Delete(P,1,1);
  I:=Pos('/',P);
  if I=0 then
    I:=Length(P);
  Delete(P,1,I);
  if (P<>'') then
    Result:=StrToIntDef(P,-1)
  else
    Result:=-2;
end;

Procedure GetRecords(aRequest : TRequest; AResponse : TResponse);

Var
  ID,First,Last : Integer;

begin
  First:=0;
  Last:=Data.Count-1;

  writeln('Commandline : ', ARequest.CommandLine);
  writeln('QueryString : ', aRequest.QueryString);

  ID:=GetRequestID(ARequest);
  writeln('ID : ', ID);
  if ID=-1 then
    begin
    Writeln('Bad GET request: ',aRequest.PATHINFO);
    InvalidParam(AResponse);
    exit;
    end;
  if (ID<>-2) then
    begin
    First:=IndexOfID(ID);
    if First=-1 then
      begin
      NotFound(AResponse);
      exit;
      end;
    Last:=First;
    SendRecords(First,Last,AResponse);
    end
  else
    begin
      writeln('QueryFields : ' , ARequest.QueryFields.Text);
      First:=StrToIntDef(ARequest.QueryFields.Values['Offset'],First);
      if (ARequest.QueryFields.Values['Limit']<>'') then
        Last:=First+StrToIntDef(ARequest.QueryFields.Values['Limit'],20)-1; // Default page size of 20
      Writeln('Count ',Data.Count,', Offset: ',First,', Last: ',Last);
      SendRecords(First,last,AResponse);
    end;
end;


Function  GetObject(aRequest : TRequest; Out AReason : String) : TJSONObject;

Var
  D : TJSONData;
  C,I : Integer;

begin
  Result:=Nil;
  if Not SameText(aRequest.ContentType,'application/json') then
    Exit;
  try
    D:=GetJSON(aRequest.Content,true);
    if (D is TJSONObject) then
      begin
      Result:=D as TJSONObject;
      C:=Result.Count-1;
      D:=Nil;
      end
    else
      begin
      FreeAndNil(D);
      end;
    I:=C;
    While Assigned(Result) and (I>=0) do
      begin
      //if (Pos(Result.Names[I]+';','id;Name;Population;')=0) then
      //  begin
      //  AReason:='Invalid property: '+Result.Names[I];
      //  FreeAndNil(Result);
      //  end;
      Dec(I);
      end;
  except
    On E : Exception do
      AReason:=E.ClassName+': '+E.Message;
  end;
end;

Procedure HandleRest(aRequest : TRequest; AResponse : TResponse);

begin
  Writeln('Method : ',ARequest.Method);
  Case UpperCase(ARequest.Method) of
    'GET': GetRecords(ARequest,AResponse);
  else
    Raise EHTTP.CreateHelp('Method not allowed',405);
  end;
end;

initialization
  LoadRestData;

finalization
  DoneRestData;
end.

